import datetime
import pytz
import uuid
from django.db import models
from django.core.validators import MaxValueValidator
from django.utils.translation import gettext_lazy as _
from tinymce.models import HTMLField
from django.conf import settings
import secrets
import string
from django.contrib.sessions.models import Session
from django.contrib.auth.hashers import check_password, make_password

class TimeStampBase(models.Model):
    date_created = models.DateTimeField(auto_now_add=True, null=True)
    date_modified = models.DateTimeField(auto_now=True, null=True)

    class Meta:
        abstract = True

class RSSFeedUrl(TimeStampBase):
    title = models.CharField(max_length=32)
    url = models.URLField()
    is_active = models.BooleanField(default=False)

    def __str__(self):
        return '%s' % self.title

    class Meta:
        verbose_name = _('RSS feed URL')
        verbose_name_plural = _('RSS feed URLs ')


class HazardLevels(models.Model):
    title = models.CharField(max_length=32)
    danger_level = models.IntegerField(validators=[MaxValueValidator(10)])
    color_code = models.CharField(max_length=6, null=True)
    description = models.TextField()

    def __str__(self):
        return '%s' % self.title

    class Meta:
        verbose_name = _('Hazard Level')
        verbose_name_plural = _('Hazard Levels')

class HazardFeeds(TimeStampBase):
    id = models.BigIntegerField(primary_key=True)
    date = models.DateTimeField()
    title = models.CharField(max_length=128)
    external_link = models.URLField()
    summary = models.TextField()
    hazard_level = models.ForeignKey(HazardLevels, on_delete=models.CASCADE)
    is_sent = models.BooleanField(default=False)
    date_send = models.DateTimeField(null=True)
    date_start = models.DateField(null=True)
    date_end = models.DateField(null=True)

    def __str__(self):
        return '%s' % self.title

    class Meta:
        verbose_name = _('Hazard Feed')
        verbose_name_plural = _('Hazard Feeds')

    def resency(self):
        now = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
        return now-self.date

    def is_newer_that(self, timedelta):
        return self.date_created-self.date < timedelta

    def date_send_set(self):
        self.date_send = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)

    @classmethod
    def not_sent(cls):
        return cls.objects.filter(is_sent=False)

def gen_act_code():
    lenth = settings.ACTIVATION_CODE_LENTH
    code = ''.join(secrets.choice(string.digits)
                   for i in range(lenth))

    return code


class ActivationCodeBaseModel(models.Model):
    """
     inherit this class to activate your model objects from sending activation codes
     activated object must have is_active field with Type models.Booleanfield
    """
    id = models.UUIDField(default=uuid.uuid4, primary_key=True)
    code = models.CharField(editable=False,  default=gen_act_code, max_length=256)
    date_created = models.DateTimeField(auto_now_add=True, null=True)


    def _is_expired(self):
        expiration = settings.CODE_EXPIRATION_TIME
        now = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
        exp = now-datetime.timedelta(seconds=expiration)
        if exp > self.date_created:
            return True
        else:
            return False

    @property
    def date_expiration(self):
        return (self.date_created + datetime.timedelta(seconds=settings.CODE_EXPIRATION_TIME)).replace(tzinfo=pytz.utc)

    def is_valid(self, code):
        if not self._is_expired():
            return code == self.code
        return False

    class Meta:
        abstract = True



class EmailActivationCode(ActivationCodeBaseModel):
    pass

class WeatherRecipients(models.Model):
    email = models.EmailField(unique=True)
    title = models.CharField(max_length=64, null=True)
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, unique=True)
    is_active = models.BooleanField(default=False)
    hazard_levels = models.ManyToManyField(HazardLevels, blank=True)
    code = models.OneToOneField(EmailActivationCode,
                                on_delete=models.SET_NULL,
                                null=True,
                                blank=True
                                )

    def __str__(self):
        return '%s' % self.title

    class Meta:
        verbose_name = _('Weather hazard recipient')
        verbose_name_plural = _('Weather hazard recipients')

class EmailTemplates(models.Model):
    title = models.CharField(max_length=64, editable=False, unique=True)
    template = HTMLField(null=True)

    def __str__(self):
        return '%s' % self.title

    class Meta:
        verbose_name = _('Email Template')
        verbose_name_plural = _('Email Templates')


class WeatherRecipientsEditCandidate(models.Model):
    title = models.CharField(max_length=64, null=True)
    valid = models.BooleanField(default=False)
    hazard_levels = models.ManyToManyField(HazardLevels, blank=True)
    target = models.OneToOneField(WeatherRecipients,
                                  on_delete=models.CASCADE,
                                  primary_key=True
                                  )


# Generated by Django 3.0.2 on 2020-01-29 10:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0003_weatherrecipients'),
    ]

    operations = [
        migrations.AddField(
            model_name='hazardfeeds',
            name='is_sent',
            field=models.BooleanField(default=False),
        ),
    ]



from django.conf import settings
from django.core.management.base import BaseCommand

from ...models import HazardFeeds
from ...utils import date_from_text_parser


def date_define():
    queryset = HazardFeeds.objects.all()
    for item in queryset:
        item.date_start, item.date_end = date_from_text_parser(settings.DATE_API, item.summary)
        print(item.date_start)
        item.save()

class Command(BaseCommand):

    def handle(self, *args, **options):
        date_define()


# Generated by Django 3.0.2 on 2020-10-26 19:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0016_auto_20201001_2227'),
    ]

    operations = [
        migrations.AddField(
            model_name='weatherrecipients',
            name='hazard_levels',
            field=models.ManyToManyField(blank=True, to='hazard_feed.HazardLevels'),
        ),
    ]



# Generated by Django 3.0.2 on 2020-10-01 19:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0015_auto_20200929_1859'),
    ]

    operations = [
        migrations.RenameField(
            model_name='hazardfeeds',
            old_name='link',
            new_name='external_link',
        ),
    ]



from django.core.management.base import BaseCommand
from hazard_feed.models import WeatherRecipients, HazardLevels


def add_hazard_levels_to_weather_recipients():
    levels = HazardLevels.objects.all().values_list('id', flat=True)
    queryset = WeatherRecipients.objects.all()
    for item in queryset:
        if not item.hazard_levels.exists():
            item.hazard_levels.add(*levels)
            item.save()

class Command(BaseCommand):

    def handle(self, *args, **options):
        add_hazard_levels_to_weather_recipients()


from django.urls import path, include
from .views.subscribe import *
from .views.warnings import *
from rest_framework.routers import DefaultRouter
app_name = 'hazard_feed'

router = DefaultRouter()
router.register(r'hazard-levels', HazardLevelsViewSet, basename='hazard_levels')
router.register(r'warnings', HazardWarningsAPIViewSet, basename='hazard_warnings')

urlpatterns = [
    path('v1/newsletter-subscribe', NewsletterSubscribeAPIView.as_view(), name='subscribe_newsletter'),
    path('v1/newsletter-unsubscribe', NewsletterUnsubscribeAPIView.as_view(), name='unsubscribe_newsletter'),
    path('v1/newsletter-subscribe/edit', NewsletterSubscribeEditApiView.as_view(), name='subscribe_edit'),
    path('v1/validate-subscribe', SubscribeValidationAPIView.as_view(), name='validate_subscribe'),
    path('v1/validate-unsubscribe', UnsubscribeValidationApiView.as_view(), name='validate_unsubscribe'),
    path('v1/validate-edit', EditValidationApiView.as_view(), name='validate_edit'),
    path('v1/weather-recipients', WeatherRecipientsRetrieveAPIView.as_view(), name='weather_recipients'),
    path('v1/', include(router.urls)),
]



from django_rq import job
import asyncio
from .utils import (
    parse_weather_feeds, put_feed_to_db,
    send_mail, get_weather_recipients, create_rss_urls_list,
    Message, send_email_async
    )

@job
def parse_feeds():
    urls_list = create_rss_urls_list()
    feeds = parse_weather_feeds(*urls_list)
    for feed in feeds:
        put_feed_to_db(feed)

@job
def send_weather_notification(feed):
    recipients = get_weather_recipients(feed)
    msg = Message.email_weather_hazard(feed)
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_until_complete(send_mail(msg, recipients))

@job
def send_email_code_activate(code, recipients):
   msg = Message.email_activation_code(code)
   send_email_async(msg, recipients)

@job
def send_email_code_deactivate(code, recipients):
    msg = Message.email_deactivation_code(code)
    send_email_async(msg, recipients)

@job
def send_email_code_edit(code, recipients):
    msg = Message.email_validate_edit_code(code)
    send_email_async(msg, recipients)




# Generated by Django 3.0.2 on 2020-09-29 15:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0014_remove_emailactivationcode_session'),
    ]

    operations = [
        migrations.AddField(
            model_name='hazardfeeds',
            name='date_end',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='hazardfeeds',
            name='date_start',
            field=models.DateField(null=True),
        ),
    ]



# Generated by Django 3.0.2 on 2020-07-31 14:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0013_emailactivationcode_is_activate'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='emailactivationcode',
            name='session',
        ),
    ]



from rest_framework import serializers
from .models import WeatherRecipients, HazardFeeds, HazardLevels
from django.conf import settings
from django.core.validators import EmailValidator


class WeatherRecipientsMailSerializer(serializers.Serializer):
    email = serializers.EmailField(required=True, allow_blank=False)

    def validate_email(self, value):
        try:
            obj = WeatherRecipients.objects.get(email=value)
        except WeatherRecipients.DoesNotExist:
            raise serializers.ValidationError(detail='email does not exist')
        return value

class SubscribeSerialiser(serializers.Serializer):
    title = serializers.CharField(required=True, allow_blank=False)
    email = serializers.EmailField(required=True, allow_blank=False, validators=[EmailValidator])
    hazard_levels = serializers.MultipleChoiceField(required=True,
                                                    allow_blank=False,
                                                    choices=HazardLevels.objects.all().values_list('id', flat=True)
                                                    )

    def create(self, validated_data):
        return validated_data

class WeatherRecipientsMailTitleSerializer(serializers.ModelSerializer):
    class Meta:
        model = WeatherRecipients
        fields = ['email', 'title']

class ActivationCodeSerializer(serializers.Serializer):
    code = serializers.CharField(required=False, allow_blank=True,
                                 max_length=settings.ACTIVATION_CODE_LENTH,
                                 min_length=settings.ACTIVATION_CODE_LENTH)
    target_uid = serializers.CharField(required=True)

    def create(self, validated_data):
        return validated_data

    def validate_code(self, value):
        try:
            code = int(value)
        except ValueError:
            raise serializers.ValidationError('Not a digit')
        return value

class SubcribeResponseSerializer(serializers.Serializer):

    expires = serializers.DateTimeField()
    target_uid = serializers.CharField()
    code_confirm = serializers.URLField()

class SuccesResponseSerializer(serializers.Serializer):

    ok = serializers.BooleanField()
    message = serializers.CharField()

class HazardLevelModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = HazardLevels
        fields = ['id', 'title', 'danger_level',
                  'color_code', 'description'
                 ]

class HazardWarningsSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="hazard_feed:hazard_warnings-detail")

    class Meta:
        model = HazardFeeds
        fields = ['id', 'url', 'title', 'external_link', 'summary',
                  'hazard_level', 'date_start', 'date_end']
        extra_kwargs = {
            'hazard_level': {'view_name': 'hazard_feed:hazard_levels-detail'},
        }

class WeatherRecipientsModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = WeatherRecipients
        fields = ['title', 'email', 'hazard_levels']


# Generated by Django 3.0.2 on 2020-03-22 15:20

from django.db import migrations, models
import django.db.models.deletion
import hazard_feed.models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('sessions', '0001_initial'),
        ('hazard_feed', '0011_rss_feed_url_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='weatherrecipients',
            name='email',
            field=models.EmailField(max_length=254, unique=True),
        ),
        migrations.CreateModel(
            name='EmailActivationCode',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('code', models.CharField(default=hazard_feed.models.gen_act_code, editable=False, max_length=256)),
                ('date_created', models.DateTimeField(auto_now_add=True, null=True)),
                ('session', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='sessions.Session')),
                ('target', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='hazard_feed.WeatherRecipients')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.0.2 on 2020-11-04 05:06

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0017_weatherrecipients_hazard_levels'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='emailactivationcode',
            name='is_activate',
        ),
        migrations.RemoveField(
            model_name='emailactivationcode',
            name='target',
        ),
        migrations.AddField(
            model_name='weatherrecipients',
            name='code',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='hazard_feed.EmailActivationCode'),
        ),
    ]



# Generated by Django 3.0.2 on 2020-02-04 20:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0004_hazardfeeds_is_sent'),
    ]

    operations = [
        migrations.AddField(
            model_name='hazardfeeds',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
        migrations.AddField(
            model_name='hazardfeeds',
            name='date_send',
            field=models.DateTimeField(null=True),
        ),
        migrations.AlterField(
            model_name='hazardfeeds',
            name='date_modified',
            field=models.DateTimeField(auto_now=True, null=True),
        ),
    ]



from rest_framework import generics, viewsets
from django.http import Http404
import django_rq
from redis.exceptions import ConnectionError
from hazard_feed.serializers import *
from rest_framework.response import Response
from rest_framework import status
from rest_framework.exceptions import ValidationError
from django.views.decorators.csrf import csrf_exempt
from hazard_feed.models import (EmailActivationCode, WeatherRecipients,
                                WeatherRecipientsEditCandidate
                                )
from django.urls import reverse_lazy
from hazard_feed.jobs import (send_email_code_activate, send_email_code_deactivate,
                              send_email_code_edit,
                              )


class NewsletterSubscribeAPIView(generics.GenericAPIView):
    serializer_class = SubscribeSerialiser
    validate_url = reverse_lazy('hazard_feed:validate_subscribe')

    def get_queryset(self):
        return WeatherRecipients.objects.all()

    def serialized_data(self, request):
        serializer = self.get_serializer(data=request.data)
        if serializer.is_valid(raise_exception=True):
            email = serializer.validated_data.get('email')
            title = serializer.validated_data.get('title')
            levels = list(serializer.validated_data.get('hazard_levels'))
            if len(levels) == 0:
                levels = list(HazardLevels.objects.all().values_list('id', flat=True))
            return email, title, levels

    def generate_code(self):
        return EmailActivationCode.objects.create()

    def create_code_response(self, code, target):
        data = {'expires': int(code.date_expiration.timestamp() * 1000),
                'target_uid': target.uuid,
                'code_confirm': self.validate_url
                }
        response_serializer = SubcribeResponseSerializer(data=data)
        response_serializer.is_valid()
        return Response(response_serializer.data, status=status.HTTP_200_OK)

    def send_notify(self, code, recipient):
        try:
            queue = django_rq.get_queue()
            queue.enqueue(send_email_code_activate, code, [recipient])
        except ConnectionError:
            send_email_code_activate(code, [recipient])

    @csrf_exempt
    def post(self, request, format=None):
        email, title, levels = self.serialized_data(request)
        queryset = self.get_queryset()
        if queryset.filter(email=email).exists():
            obj = queryset.get(email=email)
            if obj.is_active:
                return Response(status=status.HTTP_302_FOUND, data={"Error": "Email already subscribed"})
            else:
                obj.title = title
                obj.hazard_levels.clear()
                obj.hazard_levels.add(*levels)
                if obj.code:
                    obj.code.delete()
                code = self.generate_code()
                obj.code = code
                obj.save()
                self.send_notify(code.code, obj.email)
                return self.create_code_response(code, obj)
        else:
            obj = WeatherRecipients.objects.create(email=email, title=title)
            obj.hazard_levels.add(*levels)
            code = self.generate_code()
            obj.code = code
            obj.save()
            self.send_notify(code.code, obj.email)
            return self.create_code_response(code, obj)

class NewsletterUnsubscribeAPIView(NewsletterSubscribeAPIView):
    serializer_class = WeatherRecipientsMailSerializer
    validate_url = reverse_lazy('hazard_feed:validate_unsubscribe')

    def get_queryset(self):
        return WeatherRecipients.objects.all()

    def handle_exception(self, exc):
        if isinstance(exc, ValidationError) and exc.detail['email'][0] == 'email does not exist':
            exc.status_code = status.HTTP_404_NOT_FOUND
        return super().handle_exception(exc)

    def serialized_data(self, request):
        serializer = self.get_serializer(data=request.data)
        if serializer.is_valid(raise_exception=True):
            return serializer.validated_data.get('email')

    def send_notify(self, code, recipient):
        try:
            queue = django_rq.get_queue()
            queue.enqueue(send_email_code_deactivate, code, [recipient])
        except ConnectionError:
            send_email_code_deactivate(code, [recipient])

    @csrf_exempt
    def post(self, request, format=None):
        email = self.serialized_data(request)
        try:
            obj = WeatherRecipients.objects.get(email=email)
        except WeatherRecipients.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND, data={"Error": "Email not registered"})
        if obj.is_active:
            if obj.code:
                obj.code.delete()
            code = self.generate_code()
            obj.code = code
            obj.save()
            self.send_notify(code.code, obj.email)
            return self.create_code_response(code, obj)
        else:
            return Response(status=status.HTTP_404_NOT_FOUND, data={"Error": "Email not registered"})
        return Response(status=status.HTTP_400_BAD_REQUEST)


class SubscribeValidationAPIView(generics.GenericAPIView):
    serializer_class = ActivationCodeSerializer

    def response_success(self):
        message = 'Подписка на рассылку успешно активирована'
        serializer = SuccesResponseSerializer(data={'ok': True, 'message': message})
        serializer.is_valid()
        return Response(status=status.HTTP_200_OK, data=serializer.data)

    def get_serialized_data(self, request):
        serializer = self.get_serializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            code = serializer.data['code']
            uid = serializer.data['target_uid']
            return code, uid

    @csrf_exempt
    def post(self, request, format=None):
        code, uid = self.get_serialized_data(request)
        try:
            obj = WeatherRecipients.objects.get(uuid=uid)
        except WeatherRecipients.DoesNotExist:
            return Response(status=status.HTTP_400_BAD_REQUEST, data={'Error': 'Invalid uid'})
        if obj.code and obj.code.is_valid(code):
            obj.is_active = True
            obj.code.delete()
            obj.code = None
            obj.save()
            return  self.response_success()
        return Response(status=status.HTTP_400_BAD_REQUEST, data={'Error': 'Invalid Code'})

class UnsubscribeValidationApiView(SubscribeValidationAPIView):

    def response_success(self):
        message = 'Подписка на рассылку деактивирована'
        serializer = SuccesResponseSerializer(data={'ok': True, 'message': message})
        serializer.is_valid()
        return Response(status=status.HTTP_200_OK, data=serializer.data)

    @csrf_exempt
    def post(self, request, format=None):
        code, uid = self.get_serialized_data(request)
        try:
            obj = WeatherRecipients.objects.get(uuid=uid)
        except WeatherRecipients.DoesNotExist:
            return Response(status=status.HTTP_400_BAD_REQUEST, data={'Error': 'Invalid uid'})
        if obj.code and obj.code.is_valid(code):
            obj.is_active = False
            obj.save()
            obj.code.delete()
            obj.code = None
            obj.save()
            return self.response_success()
        return Response(status=status.HTTP_400_BAD_REQUEST, data={'Error': 'Invalid Code'})


class NewsletterSubscribeEditApiView(NewsletterSubscribeAPIView):
    validate_url = reverse_lazy('hazard_feed:validate_edit')

    def send_notify(self, code, recipient):
        try:
            queue = django_rq.get_queue()
            queue.enqueue(send_email_code_edit, code, [recipient])
        except ConnectionError:
            send_email_code_edit(code, [recipient])

    @csrf_exempt
    def post(self, request, format=None):
        email, title, levels = self.serialized_data(request)
        queryset = self.get_queryset()
        if queryset.filter(email=email).exists():
            obj = queryset.get(email=email)
            if obj.is_active:
                if WeatherRecipientsEditCandidate.objects.filter(target__email=email).exists():
                    WeatherRecipientsEditCandidate.objects.get(target__email=email).delete()
                candidate = WeatherRecipientsEditCandidate.objects.create(
                    target=obj,
                    title=title,
                )
                candidate.hazard_levels.add(*levels)
                candidate.save()
                if obj.code:
                    obj.code.delete()
                    obj.code = None
                code = self.generate_code()
                obj.code = code
                obj.save()
                self.send_notify(code.code, obj.email)
                return self.create_code_response(code, obj)
        return Response(status=status.HTTP_404_NOT_FOUND, data={'Error': 'Email is not registered'})


class EditValidationApiView(SubscribeValidationAPIView):

    def response_success(self):
        message = 'Параметры подписки успешно изменены'
        serializer = SuccesResponseSerializer(data={'ok': True, 'message': message})
        serializer.is_valid()
        return Response(status=status.HTTP_200_OK, data=serializer.data)

    @csrf_exempt
    def post(self, request, format=None):
        code, uid = self.get_serialized_data(request)
        try:
            obj = WeatherRecipients.objects.get(uuid=uid)
        except WeatherRecipients.DoesNotExist:
            return Response(status=status.HTTP_400_BAD_REQUEST, data={'Error': 'Invalid uid'})
        if obj.code and obj.code.is_valid(code):
            candidate = WeatherRecipientsEditCandidate.objects.get(target__uuid= uid)
            obj.title = candidate.title
            obj.hazard_levels.clear()
            obj.hazard_levels.add(*list(candidate.hazard_levels.all().values_list('id', flat=True)))
            obj.save()
            obj.code.delete()
            obj.code = None
            obj.save()
            candidate.delete()
            return self.response_success()
        return Response(status=status.HTTP_400_BAD_REQUEST, data={'Error': 'Invalid Code'})

class WeatherRecipientsRetrieveAPIView(generics.RetrieveAPIView):
    http_method_names = [u'trace', u'head', u'options', u'post']
    queryset = WeatherRecipients.objects.all().filter(is_active=True)
    serializer_class_response = WeatherRecipientsModelSerializer
    serializer_class = WeatherRecipientsMailSerializer

    def post(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

    def retrieve(self, request, *args, **kwargs):
        self.object = self.get_object()
        serializer = self.serializer_class_response(self.object)
        return Response(serializer.data)

    def get_object(self, queryset=None):
        """
        Returns the object the view is displaying.

        By default this requires `self.queryset` and a `pk` or `slug` argument
        in the URLconf, but subclasses can override this to return any object.
        """
        if queryset is None:
            queryset = self.get_queryset()

        serializer = self.get_serializer(data=self.request.data)
        serializer.is_valid(raise_exception=True)
        email = serializer.validated_data.get('email')

        if email is not None:
            queryset = queryset.filter(email=email)

        try:
            # Get the single item from the filtered queryset
            obj = queryset.get()
        except queryset.model.DoesNotExist:
            raise Http404("No %(verbose_name)s found matching the query" %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj




from rest_framework import viewsets
from rest_framework.pagination import PageNumberPagination
from ..models import  HazardFeeds, HazardLevels
from ..serializers import HazardWarningsSerializer, HazardLevelModelSerializer
from django_filters import rest_framework as filters


class HazardWarningsPageNumberPagination(PageNumberPagination):
    page_size = 5
    max_page_size = 20
    page_size_query_param = 'page_size'



class HazardLevelsViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = HazardLevels.objects.all()
    serializer_class = HazardLevelModelSerializer


class DateFilterSet(filters.FilterSet):
    date_end = filters.DateFromToRangeFilter()
    date_start = filters.DateFromToRangeFilter()

    class Meta:
        model = HazardFeeds
        fields = ['date_end', 'date_start']

class HazardWarningsAPIViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = HazardFeeds.objects.all().order_by('date').reverse()
    serializer_class = HazardWarningsSerializer
    pagination_class = HazardWarningsPageNumberPagination
    filter_backends = [filters.DjangoFilterBackend]
    filterset_class = DateFilterSet






# Generated by Django 3.0.2 on 2020-04-04 13:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0012_auto_20200322_1820'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailactivationcode',
            name='is_activate',
            field=models.BooleanField(default=True),
        ),
    ]



from django.test import TestCase, override_settings
from .utils import *
from .config import WEATHER_FEED_URL
from django.core.exceptions import ObjectDoesNotExist
import asyncio
import django_rq
from rq_scheduler import Scheduler
from .jobs import parse_feeds
from .models import WeatherRecipients
from  django.urls import reverse
from rest_framework.test import APIRequestFactory
from .views import *
from .serializers import ActivationCodeSerializer
import jwt
from rest_framework.test import APITestCase
import datetime

class TestHazardFeeds(TestCase):
    fixtures = ['hazard_feed/fixtures/hazard_levels.json']

    def setUp(self):
        pass

    def test_put_feed_to_db(self):
        feeds = parse_weather_feeds(WEATHER_FEED_URL)
        put_feed_to_db(feeds[0])
        f = HazardFeeds.objects.all()
        for item in f:
            print(item.summary)
            print(item.date_start)
            print(item.date_end)

    def test_date_compare(self):
        d1 = datetime.datetime.utcnow()
        d2 = datetime.datetime.utcnow()+datetime.timedelta(hours=3)
        d3 = datetime.datetime.utcnow()+datetime.timedelta(days=1)
        # self.assertEqual(d1.date(), d2.date())
        self.assertNotEqual(d1.date(), d3.date())


    def test_send_weather_mail(self):

        h1 = HazardLevels.objects.get(id=1)
        h2 = HazardLevels.objects.get(id=2)
        h3 = HazardLevels.objects.get(id=3)
        h4 = HazardLevels.objects.get(id=4)

        feed = HazardFeeds.objects.create(
            id=1580800025,
            date=datetime.datetime.utcnow(),
            date_modified=datetime.datetime.utcnow() + datetime.timedelta(minutes=5),
            title='Предупреждение о явлении',
            external_link='http://www.pogoda.by/news/?page=34647',
            summary='Желтый уровень опасности. 5 февраля (среда) на '
                    'отдельных участках дорог республики ожидается гололедица.',
            hazard_level=h3,
            is_sent=False
        )

        msg = make_weather_hazard_message(feed)
        recipients = get_weather_recipients(feed)
        event_loop = asyncio.get_event_loop()
        event_loop.run_until_complete(send_mail(msg, recipients))

    def test_rq(self):
        # queue = django_rq.get_queue('default')
        # queue.enqueue(parse_feeds)
        redis_conn = django_rq.get_connection
        scheduler = Scheduler(connection=redis_conn)
        scheduler.schedule(scheduled_time=datetime.datetime.utcnow()+datetime.timedelta(seconds=5),
                               func=parse_feeds,
                               interval=60
                               )
    def test_notify_signal(self):
        feed = HazardFeeds.objects.create(
            id=1580800025,
            date=datetime.datetime.utcnow(),
            date_modified=datetime.datetime.utcnow()+datetime.timedelta(minutes=5),
            title='Предупреждение о неблагоприятном явлении',
            link='http://www.pogoda.by/news/?page=34647',
            summary='Желтый уровень опасности. 5 февраля (среда) на '
                    'отдельных участках дорог республики ожидается гололедица.',
            hazard_level=HazardLevels.objects.get(id=3),
            is_sent=False
        )
        self.assertTrue(feed.is_sent)
        print(feed.date_created)
        print(feed.date_modified)

    def test_recipient(self):
        self.assertIsInstance(get_weather_recipients(), list)

    def test_templated_msg(self):
        feed = HazardFeeds(
            id=1580800025,
            date=datetime.datetime.utcnow(),
            date_modified=datetime.datetime.utcnow() + datetime.timedelta(minutes=5),
            title='Предупреждение о неблагоприятном явлении',
            link='http://www.pogoda.by/news/?page=34647',
            summary='Желтый уровень опасности. 5 февраля (среда) на '
                    'отдельных участках дорог республики ожидается гололедица.',
            hazard_level=HazardLevels.objects.get(id=3),
            is_sent=False
        )
        feed.save()


    def test_url_not_rss(self):
       print(len(parse_weather_feeds('sfsdf','http://tut.by')))


    def test_parse(self):
        list = create_rss_urls_list()
        print(list)
        feeds = parse_weather_feeds(*list)
        print(feeds)


class TestApi(APITestCase):


    def test_subscribe(self):
        pass

    def test_subscribe_edit(self):
        h1 = HazardLevels.objects.get(id=1)
        h2 = HazardLevels.objects.get(id=2)
        h3 = HazardLevels.objects.get(id=3)
        h4 = HazardLevels.objects.get(id=4)
        test = WeatherRecipients.objects.create(
            title='test',
            email='test@test.ru',
            is_active=True,
        )
        test.hazard_levels.add(h4)
        resp = self.client.post(reverse('hazard_feed:subscribe_edit'),
                                {'title': 't', 'email': 'test@test.ru', 'hazard_levels': [3, 4]},
                                format='json')
        self.assertEqual(resp.status_code, 200)
        candidate = WeatherRecipientsEditCandidate.objects.get(target__email='test@test.ru')
        self.assertEqual('t', candidate.title)
        resp = self.client.post(reverse('hazard_feed:subscribe_edit'),
                                {'title': 'test', 'email': 'test@test.ru', 'hazard_levels': [3, 4]},
                                format='json')
        self.assertEqual(resp.status_code, 200)
        test.is_active = False
        test.save()
        resp = self.client.post(reverse('hazard_feed:subscribe_edit'),
                                {'title': 'test', 'email': 'tes1t@test.ru', 'hazard_levels': [3, 4]},
                                format='json')
        self.assertEqual(resp.status_code, 404)


    def test_code_gen(self):
        resp = self.client.post(reverse('hazard_feed:activate_subscribe'),
                                {"code": "12345678"}, format='json')
        print(resp.content)

class TestUtils(APITestCase):
    fixtures = ['hazard_levels']

    def test_remove_level(self):
        text = 'Оранжевый уровень опасности. Днем 10 сентября (четверг) на большей части территории республики ожидается усиление ветра порывами до 15-20 м/с.'
        hazard_level = hazard_level_in_text_find(text)
        self.assertEqual(remove_hazard_level_from_feed(hazard_level, text), 'Днем 10 сентября (четверг) на большей части территории республики ожидается усиление ветра порывами до 15-20 м/с.' )

    def text_date_parser(self):
        text = 'Днем 10 сентября (четверг) на большей части территории республики ожидается усиление ветра порывами до 15-20 м/с.'
        date_start, date_end = date_from_text_parser('http://127.0.0.2:5000', text)
        self.assertIsNone(date_start)
        self.assertIsNone(date_end)
        date_start, date_end = date_from_text_parser('http://127.0.0.1:5000/v1/parse-date', text)
        d_s = d_n = datetime.date(2020, 9, 10)
        self.assertEqual(d_s, date_start)
        self.assertEqual(d_n, date_end)

class TestJWT(APITestCase):

    def test_jwt(self):
        now = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
        exp = now + datetime.timedelta(seconds=5)
        token = jwt.encode({'id': '5', 'exp': exp},
                           settings.SECRET_KEY, algorithm='HS256').decode('utf-8')
        serializer = ActivationCodeSerializer(data={'code': 12345678, 'token': token })
        serializer.is_valid()
        dict = jwt.decode(serializer.validated_data['token'], settings.SECRET_KEY, algorithm='HS256')
        self.assertEqual('5', dict['id'])


# Generated by Django 3.0.2 on 2020-11-04 09:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0018_auto_20201104_0806'),
    ]

    operations = [
        migrations.CreateModel(
            name='WeatherRecipientsEditCandidate',
            fields=[
                ('title', models.CharField(max_length=64, null=True)),
                ('valid', models.BooleanField(default=False)),
                ('target', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to='hazard_feed.WeatherRecipients')),
                ('hazard_levels', models.ManyToManyField(blank=True, to='hazard_feed.HazardLevels')),
            ],
        ),
    ]



# Generated by Django 3.0.2 on 2020-02-19 14:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0009_initial_email_templates_import'),
    ]

    operations = [
        migrations.CreateModel(
            name='RSSFeedUrl',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateTimeField(auto_now_add=True, null=True)),
                ('date_modified', models.DateTimeField(auto_now=True, null=True)),
                ('title', models.CharField(max_length=32)),
                ('url', models.URLField()),
                ('is_active', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'RSS feed URL',
                'verbose_name_plural': 'RSS feed URLs ',
            },
        ),
    ]



from django.contrib import admin
from .models import (
    HazardLevels, HazardFeeds,
    WeatherRecipients, EmailTemplates,
    RSSFeedUrl, EmailActivationCode
)

class BaseEnadledDisabledModelAdmin(admin.ModelAdmin):
    def make_enabled(self, request, queryset):
        rows_updated = queryset.update(is_active=True)
        if rows_updated == 1:
            message_bit = "1 запись активирована"
        else:
            message_bit = "%s записей были активированы" % rows_updated
        self.message_user(request, "%s" % message_bit)

    make_enabled.short_description = "Сделать активными"

    def make_disabled(self, request, queryset):
        rows_updated = queryset.update(is_active=False)
        if rows_updated == 1:
            message_bit = "1 запись отключена"
        else:
            message_bit = "%s записей были отключены" % rows_updated
        self.message_user(request, "%s" % message_bit)

    make_disabled.short_description = "Отключить"

    actions = [make_enabled, make_disabled]




@admin.register(HazardLevels)
class HazardLevelsAdmin(admin.ModelAdmin):
    pass

@admin.register(HazardFeeds)
class HazardFeedsAdmin(admin.ModelAdmin):
    list_display = [field.name for field in HazardFeeds._meta.get_fields()]

@admin.register(WeatherRecipients)
class WeatherRecipientsAdmin(BaseEnadledDisabledModelAdmin):
    list_display = ('title', 'email', 'is_active',)
    list_filter = ('is_active',)
    list_editable = ('is_active',)
    readonly_fields = ('id',)
    ordering = ['id']

@admin.register(EmailTemplates)
class EmailTemplateAdmin(admin.ModelAdmin):
    readonly_fields = (['title'])

@admin.register(RSSFeedUrl)
class RSSFeedUrlAdmin(BaseEnadledDisabledModelAdmin):
    list_display = ('title', 'url', 'is_active',)
    list_filter = ('is_active',)
    list_editable = ('is_active',)
    readonly_fields = ('id',)
    ordering = ['id']

@admin.register(EmailActivationCode)
class EmailActivationCodeAdmin(admin.ModelAdmin):
    list_display = [field.name for field in EmailActivationCode._meta.get_fields()]
    readonly_fields = ('code',)


from django.apps import AppConfig
from django.db.models.signals import post_save
import os
import django_rq
import datetime
from redis.exceptions import ConnectionError
from django.conf import settings



class HazardFeedConfig(AppConfig):
    """
     app settings. You must specify settings in your environment
    """
    name = 'hazard_feed'
    WEATHER_EMAIL_SMTP_HOST = os.getenv('WEATHER_EMAIL_SMTP_HOST')
    WEATHER_USE_TSL = int(os.getenv('WEATHER_USE_TSL', 0))
    WEATHER_EMAIL_SMTP_PORT = os.getenv('WEATHER_EMAIL_SMTP_PORT')
    WEATHER_EMAIL_HOST_USER = os.getenv('WEATHER_EMAIL_HOST_USER')
    WEATHER_EMAIL_HOST_PASSWORD = os.getenv('WEATHER_EMAIL_HOST_PASSWORD')


    def ready(self):

        if hasattr(settings, 'WEATHER_EMAIL_SMTP_HOST'):
            self.WEATHER_EMAIL_SMTP_HOST = settings.WEATHER_EMAIL_SMTP_HOST
        if hasattr(settings, 'WEATHER_USE_TSL'):
            self.WEATHER_USE_TSL = int(settings.WEATHER_USE_TSL)
        if hasattr(settings, 'WEATHER_EMAIL_SMTP_PORT'):
            self.WEATHER_EMAIL_SMTP_PORT = settings.WEATHER_EMAIL_SMTP_PORT
        if hasattr(settings, 'WEATHER_EMAIL_HOST_USER'):
            self.WEATHER_EMAIL_HOST_USER = settings.WEATHER_EMAIL_HOST_USER
        if hasattr(settings, 'WEATHER_EMAIL_HOST_PASSWORD'):
            self.WEATHER_EMAIL_HOST_PASSWORD = settings.WEATHER_EMAIL_HOST_PASSWORD

        from .models import HazardFeeds, EmailActivationCode
        from .signals import send_hazard_feed_notification
        post_save.connect(send_hazard_feed_notification, sender=HazardFeeds)

        from . import jobs

        try:
            scheduler = django_rq.get_scheduler('default')
            for job in scheduler.get_jobs():
                if job.func_name == 'hazard_feed.jobs.parse_feeds':
                    job.delete()
            scheduler.schedule(scheduled_time=datetime.datetime.utcnow() + datetime.timedelta(seconds=5),
                               func=jobs.parse_feeds,
                               interval=600, result_ttl=650
                               )
        except ConnectionError:
            pass


import re
import feedparser
import time
import asyncio
import requests
from .models import *
from django.conf import settings
import aiosmtplib
import nltk
import json
import dateutil.parser
from email.message import EmailMessage
from bs4 import BeautifulSoup
from django.apps import apps
from django.template import Context, Template
from django.db.utils import OperationalError
from django.core.validators import URLValidator
from django.core.exceptions import ValidationError
from .config import WEATHER_FEED_URL
from django.contrib.sessions.models import Session


def hazard_level_in_text_find(text):
    """
    chek if hazard level in text
    :param text:
    :return:
    """
    try:
        for hazard in HazardLevels.objects.all():
            if re.search(hazard.title, text):
                return hazard
    except OperationalError:
        return None
    return None


def create_rss_urls_list():
    urls = list(RSSFeedUrl.objects.filter(is_active=True).values_list('url', flat=True))
    if len(urls) == 0:
        urls.append(WEATHER_FEED_URL)
    return urls


def parse_weather_feeds(*args):
    """
    parse weather hazard rss to django model
    :param url:url of weather page
    :return:
    """
    validator = URLValidator()
    feeds_out = []
    for entry in args:
        if isinstance(entry, str):
            try:
                validator(entry)
                url = entry
                feeds = feedparser.parse(url)
                for feed in feeds.entries:
                    ms = int(time.mktime(feed.published_parsed))
                    date = datetime.datetime.fromtimestamp(ms).replace(tzinfo=pytz.utc)
                    hazard_level = hazard_level_in_text_find(feed.summary)
                    summary = remove_hazard_level_from_feed(hazard_level, feed.summary)
                    date_start, date_end = date_from_text_parser(settings.DATE_API, feed.summary)
                    if hazard_level:
                        hazard_feed = HazardFeeds(
                            id=feed.id, date=date, title=feed.title,
                            external_link=feed.link, summary=summary,
                            hazard_level=hazard_level,
                            date_start=date_start,
                            date_end=date_end,
                            is_sent=False
                        )
                        feeds_out.append(hazard_feed)
                    else:
                        raise Exception('Hazard level define error')
            except ValidationError as e:
                print(e)
                print('--'+entry+'--  not a url')

    return feeds_out

def put_feed_to_db(feed):
    try:
        if not HazardFeeds.objects.filter(id=feed.id).exists():
            feed.save()
            return True
    except OperationalError:
        return False
    return False

def make_weather_hazard_message(feed):
    date = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
    local_tz = pytz.timezone(settings.TIME_ZONE)
    date = date.astimezone(local_tz)
    template = Template(EmailTemplates.objects.get(title='weather_mail').template)
    context = Context({'date': date, 'feed': feed})
    html = template.render(context)
    soup = BeautifulSoup(html, 'html.parser')
    text = soup.get_text()
    msg = EmailMessage()
    msg['From'] = str(settings.WEATHER_EMAIL_FROM)
    msg['Subject'] = feed.title
    msg.set_content(text)
    msg.add_alternative(html, subtype='html')
    return msg



def get_weather_recipients(feed):
    level = feed.hazard_level
    recipients = list(level.weatherrecipients_set.all().values_list('email', flat=True))
    return recipients


async def send_mail(msg, recipients):

    """
    try to get queryset with async
    :param msg:
    :param recipients:
    :return:
    """
    config = apps.get_app_config('hazard_feed')
    if isinstance(recipients, list) and len(recipients) > 0:
        if config.WEATHER_USE_TSL:
            await aiosmtplib.send(
                msg,
                hostname=config.WEATHER_EMAIL_SMTP_HOST,
                port=config.WEATHER_EMAIL_SMTP_PORT,
                use_tls=config.WEATHER_USE_TSL,
                username=config.WEATHER_EMAIL_HOST_USER,
                password=config.WEATHER_EMAIL_HOST_PASSWORD,
                sender=settings.WEATHER_EMAIL_FROM,
                recipients=recipients
            )
        else:
            await aiosmtplib.send(
                msg,
                hostname=config.WEATHER_EMAIL_SMTP_HOST,
                port=config.WEATHER_EMAIL_SMTP_PORT,
                username=config.WEATHER_EMAIL_HOST_USER,
                password=config.WEATHER_EMAIL_HOST_PASSWORD,
                sender=settings.WEATHER_EMAIL_FROM,
                recipients=recipients
            )



def get_session_obj(request):
    """
    function return django session objects from request
    :param request:
    :return:
    """
    request.session.save()
    session_id = request.session.session_key
    return Session.objects.get(session_key=session_id)

class Message():

    def __init__(self):
        self.activation_template = Template(EmailTemplates.objects.get(title='activation_code_mail').template)
        self.deactivation_template = Template(EmailTemplates.objects.get(title='deactivation_code_mail').template)
        self.edit_validate_template = Template(EmailTemplates.objects.get(title='edit_validation_code_mail').template)

    @classmethod
    def email_weather_hazard(cls, feed):
        date = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
        local_tz = pytz.timezone(settings.TIME_ZONE)
        date = date.astimezone(local_tz)
        template = Template(EmailTemplates.objects.get(title='weather_mail').template)
        context = Context({'date': date, 'feed': feed})
        html = template.render(context)
        soup = BeautifulSoup(html, 'html.parser')
        text = soup.get_text()
        msg = EmailMessage()
        msg['From'] = settings.WEATHER_EMAIL_FROM
        msg['Subject'] = feed.title
        msg.set_content(text)
        msg.add_alternative(html, subtype='html')
        return msg

    def _email_code(self, code, activate=True):
        if activate:
            template = self.activation_template
        else:
            template = self.deactivation_template
        context = Context({'code': code})
        html = template.render(context)
        soup = BeautifulSoup(html, 'html.parser')
        text = soup.get_text()
        msg = EmailMessage()
        msg['From'] = settings.WEATHER_EMAIL_FROM
        if activate:
            msg['Subject'] = 'Код активации подписки'
        else:
            msg['Subject'] = 'Код деактивации подписки'
        msg.set_content(text)
        msg.add_alternative(html, subtype='html')
        return msg


    @classmethod
    def email_activation_code(cls, code):
        return cls()._email_code(code, activate=True)

    @classmethod
    def email_deactivation_code(cls, code):
        return cls()._email_code(code, activate=False)

    @classmethod
    def email_validate_edit_code(cls, code):
        template = cls().edit_validate_template
        context = Context({'code': code})
        html = template.render(context)
        soup = BeautifulSoup(html, 'html.parser')
        text = soup.get_text()
        msg = EmailMessage()
        msg['From'] = settings.WEATHER_EMAIL_FROM
        msg['Subject'] = 'Код подтверждения'
        msg.set_content(text)
        msg.add_alternative(html, subtype='html')
        return msg

def datetime_parser(json_dict):
    for (key, value) in json_dict.items():
        try:
            json_dict[key] = dateutil.parser.parse(value)
        except (ValueError, AttributeError):
            pass
    return json_dict

def date_from_text_parser(url, text):
    try:
        r = requests.post(url, data={'text': text})
        if r.status_code == 200:
            d = json.loads(r.content, object_hook=datetime_parser)
            date_start = datetime.date(d['start'].year, d['start'].month, d['start'].day)
            date_end = datetime.date(d['end'].year, d['end'].month, d['end'].day)
            return date_start, date_end
        else:
            return None, None
    except requests.exceptions.ConnectionError:
        return None, None

def send_email_async(msg, recipients):
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_until_complete(send_mail(msg, recipients))

def remove_hazard_level_from_feed(hazard_level, text):
    result = ''
    sentences = nltk.sent_tokenize(text)
    if hazard_level == None:
        return text
    for sentence in sentences:
        if not re.search(hazard_level.title, sentence):
            result += sentence
    return result





import os
from sys import path

from django.db import models, migrations
from django.core import serializers

fixture_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '../fixtures'))
fixture_filename = 'rss_urls.json'

def deserialize_fixture():
    fixture_file = os.path.join(fixture_dir, fixture_filename)

    with open(fixture_file, 'rb') as fixture:
        return list(serializers.deserialize('json', fixture, ignorenonexistent=True))


def load_fixture(apps, schema_editor):
    objects = deserialize_fixture()

    for obj in objects:
        obj.save()


def unload_fixture(apps, schema_editor):
    """Delete all EmailTemplates objects"""

    objects = deserialize_fixture()

    RSSFeedUrl = apps.get_model("hazard_feed", "RSSFeedUrl")
    RSSFeedUrl.objects.filter(pk__in=[obj.object.pk for obj in objects]).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0010_rssfeedurl'),
    ]

    operations = [
        migrations.RunPython(load_fixture, reverse_code=unload_fixture),
    ]


# Generated by Django 3.0.2 on 2020-01-26 14:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='hazardlevels',
            name='color_code',
            field=models.CharField(max_length=6, null=True),
        ),
    ]
